import { __decorate, __metadata } from "tslib";
import { inject, singleton } from 'mana-syringe';
import { ComponentManager } from './ComponentManager';
import { EntityManager } from './EntityManager';
import { IDENTIFIER } from './identifier';
import { SystemManager } from './SystemManager';
/**
 * the context of ECS
 * @see https://github.com/sschmid/Entitas-CSharp/blob/master/Entitas/Entitas/Context/IContext.cs
 * @example
 * ```
 * // register components
 * world
    .registerComponent(C1)
    .registerComponent(C2)
    .registerComponent(C3);
  
  // register systems
 * ```
 */

var World =
/** @class */
function () {
  function World() {
    this.lastMillis = new Date().getTime();
    this.enabled = true;
  }

  World.prototype.createEntity = function (name) {
    if (name === void 0) {
      name = '';
    }

    var entity = this.entityPoolFactory().acquire();
    entity.setName(name);
    return this.entityManager.createEntity(entity);
  };

  World.prototype.getEntityByName = function (name) {
    return this.entityManager.getEntityByName(name);
  };

  World.prototype.registerComponent = function (clazz) {
    this.componentManager.registerComponent(clazz);
    return this;
  };

  World.prototype.registerSystem = function (clazz) {
    this.systemManager.registerSystem(clazz);
    return this;
  };

  World.prototype.execute = function (delta, millis) {
    if (!delta) {
      millis = new Date().getTime();
      delta = millis - this.lastMillis;
      this.lastMillis = millis;
    }

    if (this.enabled) {
      this.systemManager.execute(delta, millis);
    }
  };

  World.prototype.stop = function () {
    this.enabled = false;
  };

  World.prototype.resume = function () {
    this.enabled = true;
  };

  World.prototype.destroy = function () {
    this.systemManager.destroy();
    this.entityManager.destroy(); // this.componentManager.destroy();
  };

  __decorate([inject(IDENTIFIER.EntityPoolFactory), __metadata("design:type", Function)], World.prototype, "entityPoolFactory", void 0);

  __decorate([inject(EntityManager), __metadata("design:type", EntityManager)], World.prototype, "entityManager", void 0);

  __decorate([inject(ComponentManager), __metadata("design:type", ComponentManager)], World.prototype, "componentManager", void 0);

  __decorate([inject(SystemManager), __metadata("design:type", SystemManager)], World.prototype, "systemManager", void 0);

  World = __decorate([singleton()], World);
  return World;
}();

export { World };