"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EntityManager = exports.ENTITY_EVENT = exports.COMPONENT_EVENT = void 0;

var _tslib = require("tslib");

var _eventemitter = require("eventemitter3");

var _manaSyringe = require("mana-syringe");

var _identifier = require("./identifier");

var ENTITY_EVENT = {
  Created: 'Created'
};
exports.ENTITY_EVENT = ENTITY_EVENT;
var COMPONENT_EVENT = {
  Added: 'Added',
  Remove: 'Remove'
};
exports.COMPONENT_EVENT = COMPONENT_EVENT;

var EntityManager =
/** @class */
function (_super) {
  (0, _tslib.__extends)(EntityManager, _super);

  function EntityManager() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.entities = [];
    _this.entitiesByNames = {};
    /**
     * use cache when query a matcher
     */

    _this.matcherCache = {};
    return _this;
  }

  EntityManager.prototype.getEntityByName = function (name) {
    return this.entitiesByNames[name];
  };

  EntityManager.prototype.getAllEntities = function () {
    return this.entities;
  };

  EntityManager.prototype.queryByMatcher = function (matcher) {
    var hash = matcher.hash();

    if (!this.matcherCache[hash]) {
      this.matcherCache[hash] = this.entities.filter(function (entity) {
        return matcher.matches(entity);
      });
    }

    return this.matcherCache[hash];
  };

  EntityManager.prototype.createEntity = function (entity) {
    var name = entity.getName();

    if (name) {
      if (this.entitiesByNames[name]) {
        throw new Error("Entity name '" + name + "' already exist");
      } else {
        this.entitiesByNames[name] = entity;
      }
    }

    this.entities.push(entity);
    this.emit(ENTITY_EVENT.Created, entity);
    return entity;
  };

  EntityManager.prototype.destroyEntity = function (entity) {
    if (entity.getName()) {
      delete this.entitiesByNames[entity.getName()];
    }

    var index = this.entities.indexOf(entity);

    if (index > -1) {
      this.entities.splice(index, 1);
    }

    this.matcherCache = {};
  };

  EntityManager.prototype.addComponentToEntity = function (entity, clazz, values) {
    var tag = clazz.tag;
    var components = entity.getComponents();
    var component = components[tag];

    if (component) {
      if (!entity.cast(component, clazz)) {
        throw new Error("There are multiple classes with the same tag or name \"" + tag + "\".\nAdd a different property \"tag\" to one of them.");
      }

      delete components[tag];
    }

    component = this.componentPoolFactory(clazz).acquire();

    if (values) {
      component.copy(values);
    }

    components[tag] = component;
    this.emit(COMPONENT_EVENT.Added, entity, component);
    this.matcherCache = {};
    return component;
  };

  EntityManager.prototype.removeComponentFromEntity = function (entity, clazz, immediately) {
    if (immediately === void 0) {
      immediately = false;
    }

    if (!entity.hasComponent(clazz)) {
      return;
    }

    this.emit(COMPONENT_EVENT.Remove, entity, clazz);

    if (immediately) {
      this.removeComponentFromEntitySync(entity, clazz);
    } else {//   if (entity._ComponentTypesToRemove.length === 0)
      //     this.entitiesWithComponentsToRemove.push(entity);
      //   entity._ComponentTypes.splice(index, 1);
      //   entity._ComponentTypesToRemove.push(Component);
      //   entity._componentsToRemove[Component._typeId] =
      //     entity._components[Component._typeId];
      //   delete entity._components[Component._typeId];
      // }
      // Check each indexed query to see if we need to remove it
      // this._queryManager.onEntityComponentRemoved(entity, Component);
      // if (Component.__proto__ === SystemStateComponent) {
      //   entity.numStateComponents--;
      //   // Check if the entity was a ghost waiting for the last system state component to be removed
      //   if (entity.numStateComponents === 0 && !entity.alive) {
      //     entity.remove();
      //   }
    }

    this.matcherCache = {};
  };

  EntityManager.prototype.destroy = function () {
    this.entities = [];
    this.entitiesByNames = {};
  };

  EntityManager.prototype.removeComponentFromEntitySync = function (entity, clazz) {
    // Remove T listing on entity and property ref, then free the component.
    // entity._ComponentTypes.splice(index, 1);
    var component = entity.getComponent(clazz);
    var components = entity.getComponents();
    delete components[clazz.tag];
    component.destroy(); // this.world.componentsManager.componentRemovedFromEntity(Component);
  };

  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_identifier.IDENTIFIER.ComponentPoolFactory), (0, _tslib.__metadata)("design:type", Function)], EntityManager.prototype, "componentPoolFactory", void 0);
  EntityManager = (0, _tslib.__decorate)([(0, _manaSyringe.singleton)()], EntityManager);
  return EntityManager;
}(_eventemitter.EventEmitter);

exports.EntityManager = EntityManager;